<?php

/**
 * @file
 * The endpoints for Desk-Net RESTful API.
 */

namespace Drupal\desk_net\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\node\Entity\Node;
use Drupal\desk_net\DeleteMethods;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller routines for test_api routes.
 */
class APIController extends ControllerBase {

  protected $api_key;

  protected $api_secret;

  public function __construct() {
    $config = \Drupal::config('desk_net.settings');
    $this->api_key = $config->get('desk_net_api_key');
    $this->api_secret = $config->get('desk_net_api_secret');
  }

  /**
   * Perform get Status list from Desk-Net.
   *
   * @param Request $request
   *   The request object.
   *
   * @return string
   *   The status saving statuses.
   */
  public function saveStatuses(Request $request) {
    $response = new Response();
    // Token validation.
    if (!$this->isTokenValid($request)) {
      return $response->setStatusCode(Response::HTTP_UNAUTHORIZED);
    }
    // This condition checks the `Content-type` and makes sure to
    // decode JSON string from the request body into array.
    if (str_starts_with($request->headers->get('Content-Type'), 'application/json')) {
      $data = json_decode($request->getContent(), TRUE);
      $request->request->replace(is_array($data) ? $data : []);
    }
    else {
      return $response->setStatusCode(Response::HTTP_NO_CONTENT);
    }

    $save_statuses_list = \Drupal::config('desk_net.settings')
      ->get('desk_net_activate_status_list');

    if (isset($data['platform'])) {
      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('platform_id', $data['platform'])
        ->save();
    }
    else {
      return $response->setStatusCode(Response::HTTP_NO_CONTENT);
    }

    $json_request = ModuleSettings::checkTriggersExportStatus($data);

    if (!empty($save_statuses_list)) {
      $save_statuses_list = unserialize($save_statuses_list);
      $drupal_status_list = ['published', 'unpublished'];

      DeleteMethods::shapeDeletedItems('desk_net_selected_status_matching_list',
        $json_request['activeStatuses'], $save_statuses_list, $drupal_status_list, 'status', FALSE);
    }
    if (!empty($json_request['activeStatuses'])) {
      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('desk_net_activate_status_list', serialize($json_request['activeStatuses']))
        ->save();

      $field_name = ActionController::generateFieldMatchingName(
        'status', 'desk_net_to_drupal', '5');
      ActionController::setFieldMatchingValue('desk_net_selected_status_matching_list', $field_name, '1');

      $field_name = ActionController::generateFieldMatchingName(
        'status', 'drupal_to_desk_net', '1');
      ActionController::setFieldMatchingValue('desk_net_selected_status_matching_list', $field_name, '5');
    }
    if (!empty($json_request['deactivatedStatuses'])) {
      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('desk_net_deactivate_status_list', serialize($json_request['deactivatedStatuses']))
        ->save();
    }

    return $response->setStatusCode(Response::HTTP_OK);
  }

  /**
   * Perform crete article in Drupal from Desk-Net.
   *
   * @param Request $request
   *   The request object.
   *
   * @return string
   *   The result creating article in Drupal from Desk-Net.
   */
  public function createPublication(Request $request) {
    $response = new Response();
    // Token validation.
    if (!$this->isTokenValid($request)) {
      return $response->setStatusCode(Response::HTTP_UNAUTHORIZED);
    }

    if (str_starts_with($request->headers->get('Content-Type'), 'application/json')) {
      $data = json_decode($request->getContent(), TRUE);
      $request->request->replace(is_array($data) ? $data : []);
    }

    if (empty($data)) {
      return FALSE;
    }

    $response = ActionController::createNode($data);

    return new JsonResponse($response);
  }

  /**
   * Perform update article in Drupal from Desk-Net.
   *
   * @param RouteMatchInterface $route_match
   *   The node id which should updating.
   * @param Request $request
   *   The request object.
   *
   * @return string
   *   The result updating article in Drupal from Desk-Net.
   */
  public function updatePublication(RouteMatchInterface $route_match, Request $request) {
    $response = new Response();
    // Token validation.
    if (!$this->isTokenValid($request)) {
      return $response->setStatusCode(Response::HTTP_UNAUTHORIZED);
    }

    $entity_id = $route_match->getRawParameter('story_id');
    $sync_updates_from_desk_net = \Drupal::config('desk_net.settings')
      ->get('sync_updates_from_desk_net');

    if ($sync_updates_from_desk_net || $sync_updates_from_desk_net === NULL) {
      if (str_starts_with($request->headers->get('Content-Type'), 'application/json')) {
        $data = json_decode($request->getContent(), TRUE);
        $request->request->replace(is_array($data) ? $data : []);
      }

      if (empty($data) || empty($entity_id) || !is_numeric($entity_id)) {
        return $response->setStatusCode(Response::HTTP_ACCEPTED);
      }

      // Loading node by id.
      $node = Node::load($entity_id);
      if ($node !== NULL) {
        // Skipping updating slug if node was published.
        if ($node->status->value == 1 && isset($data['slug'])) {
          unset($data['slug']);
        }

        // Getting Desk-Net revision data.
        $desk_net_revision = ModuleSettings::deskNetRevisionGet($node);
        // Check node - removed status.
        if ($desk_net_revision !== FALSE && $desk_net_revision['desk_net_removed_status'] == 'desk_net_removed') {
          return $response->setStatusCode(Response::HTTP_ACCEPTED);
        }
      }
      else {
        return $response->setStatusCode(Response::HTTP_NOT_FOUND);
      }

      $response = ActionController::updateNode($data, $entity_id, $desk_net_revision);

      return new JsonResponse($response);
    }
    else {
      return new JsonResponse(ActionController::generate_drupal_response_data($entity_id));
    }
  }

  /**
   * Perform delete node in Drupal by Desk-Net.
   *
   * @param RouteMatchInterface $route_match
   *   The node id for delete.
   * @param Request $request
   *   The request object.
   *
   * @return array|bool
   *   The result updating article status on 'Deleted/Removed' in Drupal.
   */
  public function deletePublication(RouteMatchInterface $route_match, Request $request) {
    $response = new Response();
    // Token validation.
    if (!$this->isTokenValid($request)) {
      return $response->setStatusCode(Response::HTTP_UNAUTHORIZED);
    }

    $sync_deletions_from_desk_net = \Drupal::config('desk_net.settings')
      ->get('sync_deletions_from_desk_net');

    if ($sync_deletions_from_desk_net || $sync_deletions_from_desk_net === NULL) {
      // Getting cms_id.
      $node_id = $route_match->getRawParameter('story_id');

      // Getting value for matching "Deleted/Removed" on page "Status Matching".
      $field_name = ActionController::generateFieldMatchingName(
        'status', 'desk_net_to_drupal', 'desk_net_removed');
      $matching_status_remove = ActionController::getFieldMatchingValue(
        'desk_net_selected_status_matching_list', $field_name);

      if ($matching_status_remove === NULL) {
        $matching_status_remove = 0;
      }
      // Loading node by id.
      $node = Node::load($node_id);

      if ($node == NULL) {
        return $response->setStatusCode(Response::HTTP_NOT_FOUND);
      }
      // Getting Desk-Net revision data.
      $desk_net_revision = ModuleSettings::deskNetRevisionGet($node);
      // Adding information for the node about deleting story in Desk-Net app.
      $desk_net_revision['desk_net_removed_status'] = 'desk_net_removed';
      $node = ModuleSettings::deskNetRevisionSet($node, $desk_net_revision);
      // Updating node status.
      $node->set('status', $matching_status_remove);

      // Saving changed.
      $node->save();
    }

    return $response->setStatusCode(Response::HTTP_OK);
  }

  /**
   * Validating authorization Token.
   *
   * @param object $request
   *   The token.
   *
   * @return boolean
   *   The result of checking.
   */
  protected function isTokenValid($request): bool {
    try {
      $token = $request->headers->get('authorization');

      if (empty($token)) {
        throw new \InvalidArgumentException("The client has not transmitted the token in the request.");
      }

      $tokenInfo = APIController::getTokenInfo($token);

      if (is_array($tokenInfo) && !self::validateTokenInfo($tokenInfo)) {
        throw new \InvalidArgumentException("Invalid token or expired token.");
      }

      return TRUE;
    }
    catch (\Exception $e) {
      \Drupal::logger('access denied')->warning($e->getMessage());
      return FALSE;
    }
  }

  /**
   * Validate token information.
   *
   * @param array $tokenInfo
   *   The token information array.
   *
   * @return bool
   *   TRUE if the token information is valid, FALSE otherwise.
   */
  public static function validateTokenInfo(array $tokenInfo): bool {
    $current_date_time = date('Y-m-d H:i:s');
    $expiration_date = $tokenInfo['expire_date'] ?? 0;

    return $expiration_date >= $current_date_time;
  }

  /**
   * Perform generate new Desk-Net module Token.
   *
   * @param Request $request
   *  The request object.
   *
   * @return string
   *   The status saving statuses.
   */
  public function generateAccessToken(Request $request) {
    $api_key = $request->headers->get('php-auth-user');
    $api_secret = $request->headers->get('php-auth-pw');

    if ($this->isValidDrupalCredentials($api_key, $api_secret)) {
      $token = DrupalCredentialsController::generateDrupalModuleToken();
      $response = new JsonResponse([
        'access_token' => $token,
        'token_type' => 'Bearer',
        'expires_in' => 300,
      ], 200, ['Content-Type' => 'application/json']);
    }
    else {
      $response = new JsonResponse([
        'message' => \Drupal::translation()->translate('Wrong credentials'),
      ], 401, ['Content-Type' => 'application/json']);
    }

    return $response;
  }

  /**
   * Checks if the provided API key and API secret match the stored credentials.
   *
   * @param string|null $api_key The API key provided.
   * @param string|null $api_secret The API secret provided.
   *
   * @return bool TRUE if the provided credentials match, FALSE otherwise.
   */
  protected function isValidDrupalCredentials($api_key, $api_secret): bool {
    if (NULL === $api_key || NULL === $api_secret) {
      return FALSE;
    }

    return $this->api_key === $api_key && $this->api_secret === $api_secret;
  }

  /**
   * Get token information from the custom table by token value.
   *
   * @param string $token
   *   The value of the token.
   *
   * @return array|null
   *   The token information array or NULL if not found.
   */
  protected static function getTokenInfo($token) {
    $token = hash('sha256', $token);
    $database = \Drupal::database();
    $table = 'desk_net_tokens';

    // Perform a database query to fetch token information.
    $query = $database->select($table, 't')
      ->fields('t')
      ->condition('t.token', $token)
      ->execute();

    $result = $query->fetchAssoc();

    return $result ?: NULL;
  }

}
